library INTLOG;

// Integer logic functions for
// ProfiLab Expert V3.0 or higher
// (www.abacom-online.de)
// Feedback to: Expert.Profi@gmx.de

uses
  Math,
  SysUtils,
  Windows,
  Classes,
  Controls,
  Dlg_IntLog in 'Dlg_IntLog.pas' {ConfigDlg};

{$R *.RES}

Const  Inputs = 2;  // number of inputs
       Outputs = 2; // number of outputs

       {DLL INPUTS}

       var_A = 0; //Operation Param A
       var_B = 1; //Operation Param B

       {DLL OUTPUTS}

       OP_Result = 0; //Operation result
       OP_Zero   = 1; //Operation result = 0

       {USER}
       OP_width  = 0; //Number of bits considered
       OP_type   = 1; //Function to perform
       OP_signed = 2; //signed or unsigned operation result


Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=Inputs; //Define number of component input pins
end;

function NumOutputs: Byte;
begin
  result:=Outputs; //Define number of component output pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   Var_A: result:='A';
   Var_B: result:='B';
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   case Channel of
   OP_Result: result:='Z';
   OP_ZERO: result:='Z=0';
   end;
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
begin
   //Init bit width as byte (first time after import)
   if PUser^[OP_width]=0 then PUser^[OP_width]:=8;
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
   //not needed
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var VarA,VarB,VarZ: DWord;
    A,B: Extended;
begin
   // Get param A / range check
   A:=PInput^[Var_A];
   //32 bit limit
   if A<$80000000/-1 then A:=$80000000/-1;
   if A> $FFFFFFFF then A:= $FFFFFFFF;
   // negative or positive
   if A<0 then VarA:=not DWord(round(-A-1)) else VarA:=DWord(round(A));
   // cut unwanted bits
   VarA:=VarA and round(Power(2,PUser^[op_width])-1);

   // Get param B / range check
   B:=PInput^[Var_B];
   //32 bit limit
   if B<$80000000/-1 then B:=$80000000/-1;
   if B>$FFFFFFFF then B:=$FFFFFFFF;
   // negative or positive
   if B<0 then VarB:=not DWord(round(-B-1)) else VarB:=DWord(round(B));
   // cut unwanted bits
   VarB:=VarB and round(Power(2,PUser^[op_width])-1);

   //perform the selected function
   case round(PUser^[OP_type]) of
   0: VarZ:=not VarA;
   1: VarZ:=VarA and VarB;
   2: VarZ:=VarA or VarB;
   3: VarZ:=not (VarA and VarB);
   4: VarZ:=not (VarA or VarB);
   5: VarZ:=VarA xor VarB;
   6: VarZ:=( VarA shl (round(PUser^[op_width]) div 2) ) or ( VarB and (round(Power(2,PUser^[op_width]/2))-1) );
   7: VarZ:=(not VarA)+1;
   else
      VarZ:=0;
   end;

   // return result
   POutput^[OP_Result]:=VarZ and (round(Power(2,PUser^[op_width]))-1);

   // Test and return "result=0"
   if (abs(POutput^[OP_Result])<0.5) then POutput^[op_Zero]:=5 else POutput^[OP_Zero]:=0;

   //signed result
   if PUser^[OP_signed]>2.5 then
   begin
      //MSB set -> negative result
      if (VarZ and (1 shl (round(PUser^[op_width])-1)))<>0 then
      begin
         //2s complement
         VarZ:= ((not VarZ)+1) and round(Power(2,PUser^[op_width])-1);
         //return negativ result
         POutput^[OP_Result]:=-(VarZ/1);
      end;
   end;

end;

Procedure Configure(PUser: PDLLParams);
var DLG: TConfigDlg;
begin
   //Init bit width as byte (first time after import)
   if PUser^[OP_width]=0 then PUser^[OP_width]:=8;
   // Create user dialogue
   DLG:=TConfigDlg.Create(nil);
   With DLG do
   begin
      // number of bits
      SpinEdit1.Value:=round(PUser^[OP_width]);
      // function type
      Combobox1.itemIndex:=round(PUser^[OP_type]);
      // signed or unsigned
      if PUser^[OP_signed]>2.5 then cb_Signed.checked:=true else cb_unsigned.checked:=true;
      // Execute dialogue
      if ShowModal=mrOK then
      begin
         // Read values from dialogue
         PUser^[OP_type]:=Combobox1.ItemIndex;
         PUser^[OP_width]:=SpinEdit1.Value;
         if cb_Signed.checked then PUser^[OP_signed]:=5 else PUser^[OP_signed]:=0;
      end;
      // Free dialogue
      free;
   end;
end;

//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;
begin
end.

